<?php
/**
* 2007-2022 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2022 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

define('_MSG_TYPE_ADMIN_', 'ADMIN');
define('_MSG_TYPE_CUSTOMER_', 'CUSTOMER');
define('_MSG_TYPES_',[_MSG_TYPE_ADMIN_,_MSG_TYPE_CUSTOMER_]);
define('_CHANNEL_OWN_DEVICES_', 0);
define('_CHANNEL_SMS_ADVERT_', 1);
define('_API_URL', 'https://www.smsadvert.ro/api/sms/');

class Smsadvert extends Module
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'smsadvert';
        $this->tab = 'administration';
        $this->version = '1.1.0';
        $this->author = 'SMSAdvert';
        $this->need_instance = 0;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Order Status SMS');
        $this->description = $this->l('Send personalized SMS notification when order status changes for both administrator and customers');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall SMSAdvert?');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    public function install()
    {
        include(dirname(__FILE__).'/sql/install.php');

        return parent::install() &&
            $this->registerHook('displayBackOfficeHeader') &&
            $this->registerHook('actionOrderStatusUpdate') &&
            $this->registerHook('actionValidateOrder');
    }

    public function uninstall()
    {
        Configuration::deleteByName('SMSADVERT_CHANNEL');
        Configuration::deleteByName('SMSADVERT_FAILOVER');
        Configuration::deleteByName('SMSADVERT_API_AUTH_TOKEN');
        Configuration::deleteByName('SMSADVERT_ADMIN_CONTACT');
        
        $orderStates = OrderState::getOrderStates($this->context->language->id);
        foreach (_MSG_TYPES_ as $type) {
            Configuration::deleteByName('SMSADVERT_NEW_ORDER_MSG_'.$type);
            Configuration::deleteByName('SMSADVERT_NEW_ORDER_MSG_'.$type.'_ACTIVE');
            foreach ($orderStates as $state) {
                Configuration::deleteByName('SMSADVERT_STATUS_MSG_'.$type.'_'.$state['id_order_state']);
                Configuration::deleteByName('SMSADVERT_STATUS_MSG_'.$type.'_ACTIVE_'.$state['id_order_state']);
            }
        }

        include(dirname(__FILE__).'/sql/uninstall.php');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitSmsadvertModule')) == true) {
            $this->postProcess();
        }

        $orderStates = OrderState::getOrderStates($this->context->language->id);
        
        $this->context->smarty->assign('orderStates', $orderStates);
        $this->context->smarty->assign('fields_value', $this->getConfigFormValues());

        return $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitSmsadvertModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        $inputFields = [];
        
        $channels = [
            [
                'id_channel' => _CHANNEL_OWN_DEVICES_,
                'name' => $this->l('Own Devices'),
            ],
            [
                'id_channel' => _CHANNEL_SMS_ADVERT_,
                'name'  =>  $this->l('SMSAdvert Network'),
            ]
        ];

        array_push($inputFields,
            [
                'type' => 'select',
                'label' => $this->l('Choose Channel'),
                'name' => 'SMSADVERT_CHANNEL',
                'desc' => $this->l('Choose Channel for sending sms.'),
                'options' => [
                    'query' => $channels,
                    'id' => 'id_channel',
                    'name' => 'name',
                ],
                'tab' => 'api_config',
            ],
            [
                'type' => 'switch',
                'label' => $this->l('Activate Failover'),
                'name' => 'SMSADVERT_FAILOVER',
                'is_bool' => true,
                'desc' => $this->l('Set Failover to Yes if you want to send through SmsAdvert Network as a secondary channel, in case your own devices are down. It works only if you select Own Devices as sending channel.'),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'label' => $this->l('Yes')
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'label' => $this->l('No')
                    ]
                ],
                'tab' => 'api_config',
            ],
            [
                'col' => 6,
                'type' => 'text',
                'prefix' => '<i class="icon icon-key"></i>',
                'desc' => $this->l('Add your SMSAdvert API Authorization Token Here, you can find it in your SMSAdvert account in the API section.'),
                'name' => 'SMSADVERT_API_AUTH_TOKEN',
                'label' => $this->l('Authorization Token'),
                'tab' => 'api_config',
            ],
            [
                'col' => 3,
                'type' => 'text',
                'prefix' => '<i class="icon icon-phone"></i>',
                'desc' => $this->l('Add Admin Phone Number here.'),
                'name' => 'SMSADVERT_ADMIN_CONTACT',
                'label' => $this->l('Admin Contact Number'),
                'tab' => 'api_config',
            ]
        );
        
        $orderStates = OrderState::getOrderStates($this->context->language->id);
        foreach (_MSG_TYPES_ as $type) {
            $inputFields[] = array(
                'type' => 'html',
                'name' => 'template_tags',
                'html_content' => $this->getTemplateTagsHTML(),
                'tab'   => $type.'_MSG_CONFIG',
            );

            $inputFields[] = array(
                'type'  =>  'text',
                'label' =>  $this->l('New Order'),
                'name'  =>  'SMSADVERT_NEW_ORDER_MSG_'.$type,               
                'class' =>  'fixed-width-xxl',
                'tab'   =>  $type.'_MSG_CONFIG',
            );
            foreach ($orderStates as $state) {
                array_push($inputFields, array(
                    'type'  =>  'text',
                    'label' =>  $state['name'],
                    'name'  =>  'SMSADVERT_STATUS_MSG_'.$type.'_'.$state['id_order_state'],               
                    'class' =>  'fixed-width-xxl',
                    'tab'   =>  $type.'_MSG_CONFIG',
                ));
            }
        }

        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'tabs' => array(
                    'api_config' => $this->l('API Configuration'),
                    'ADMIN_MSG_CONFIG' => $this->l('Admin Messages'),
                    'CUSTOMER_MSG_CONFIG' => $this->l('Customer Messages'),
                ),
                'input' => $inputFields,
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        $formValues = array();
        $orderStates = OrderState::getOrderStates($this->context->language->id);

        $formValues['SMSADVERT_CHANNEL'] = Configuration::get('SMSADVERT_CHANNEL', null);
        $formValues['SMSADVERT_FAILOVER'] = Configuration::get('SMSADVERT_FAILOVER', null);
        $formValues['SMSADVERT_API_AUTH_TOKEN'] = Configuration::get('SMSADVERT_API_AUTH_TOKEN', null);
        $formValues['SMSADVERT_ADMIN_CONTACT'] = Configuration::get('SMSADVERT_ADMIN_CONTACT', null);

        foreach (_MSG_TYPES_ as $type) {
            $formValues['SMSADVERT_NEW_ORDER_MSG_'.$type] = Configuration::get('SMSADVERT_NEW_ORDER_MSG_'.$type);
            $formValues['SMSADVERT_NEW_ORDER_MSG_'.$type.'_ACTIVE'] = Configuration::get('SMSADVERT_NEW_ORDER_MSG_'.$type.'_ACTIVE');
            foreach ($orderStates as $key => $state) {
                $formValues['SMSADVERT_STATUS_MSG_'.$type.'_'.$state['id_order_state']] = Configuration::get('SMSADVERT_STATUS_MSG_'.$type.'_'.$state['id_order_state']);
                $formValues['SMSADVERT_STATUS_MSG_'.$type.'_ACTIVE_'.$state['id_order_state']] = Configuration::get('SMSADVERT_STATUS_MSG_'.$type.'_ACTIVE_'.$state['id_order_state']);
            }
        }
        
        return $formValues;
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
    * Add the CSS & JavaScript files you want to be loaded in the BO.
    */
    public function hookDisplayBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name || Tools::getValue('configure') == $this->name) {
            $this->context->controller->addJS($this->_path.'views/js/back.js');
            $this->context->controller->addCSS($this->_path.'views/css/back.css');
        }
    }

    public function hookActionValidateOrder($params)
    {
        $order = $params['order'];
        $address_delivery = new Address($order->id_address_delivery);
        $address_invoice = new Address($order->id_address_invoice);
        $phone_mobile = $address_delivery->phone_mobile;
        $phone_home = $address_delivery->phone;
        $customer = $params['customer'];
        
        foreach (_MSG_TYPES_ as $type) {
            $isStatusMsgEnabled = Configuration::get('SMSADVERT_NEW_ORDER_MSG_'.$type.'_ACTIVE');
            if (!empty($isStatusMsgEnabled) && $isStatusMsgEnabled) {
                $msg = Configuration::get('SMSADVERT_NEW_ORDER_MSG_'.$type);
                $msg = $this->setVariableValues($msg, $order, $customer, $address_delivery->address1, $address_invoice->address1);          
                $phone = ($type == _MSG_TYPE_ADMIN_)
                    ? Configuration::get('SMSADVERT_ADMIN_CONTACT')
                    : (empty($phone_mobile)?$phone_home:$phone_mobile);

                $response = $this->sendSMSRequest($phone, $msg);
                if (isset($response->successMessage)) {
                    $this->addMessageLog($phone, $msg, $response->msgId);
                }
            }
        }
    }

    public function hookActionOrderStatusUpdate($params)
    {
        $order = new Order((int) $params['id_order']);
        $address_delivery = new Address($order->id_address_delivery);
        $address_invoice = new Address($order->id_address_invoice);
        $phone_mobile = $address_delivery->phone_mobile;
        $phone_home = $address_delivery->phone;
        $customer = new Customer($order->id_customer);
        $orderStatus = $params['newOrderStatus'];
        
        foreach (_MSG_TYPES_ as $type) {
            $isStatusMsgEnabled = Configuration::get('SMSADVERT_STATUS_MSG_'.$type.'_ACTIVE_'.$orderStatus->id);
            if (!empty($isStatusMsgEnabled) && $isStatusMsgEnabled) {
                $msg = Configuration::get('SMSADVERT_STATUS_MSG_'.$type.'_'.$orderStatus->id);
                $msg = $this->setVariableValues($msg, $order, $customer, $address_delivery->address1, $address_invoice->address1);             
                $phone = ($type == _MSG_TYPE_ADMIN_)
                    ? Configuration::get('SMSADVERT_ADMIN_CONTACT')
                    : (empty($phone_mobile)?$phone_home:$phone_mobile);

                $response = $this->sendSMSRequest($phone, $msg);
                if (isset($response->successMessage)) {
                    $this->addMessageLog($phone, $msg, $response->msgId);
                }
            }
        }
    }

    /**
     * Generate HTML for tags field
     */
    public function getTemplateTagsHTML()
    {
        $html = '</div>';
        $html .= '<label>Note: Given tags can be used in message templates (Click to Copy)</label><br/>';
        $html .= '<center>';
        $html .= '<span class="label label-primary tag" onclick="copyToClipboard(this)">{id_order}</span> ';
        $html .= '<span class="label label-primary tag" onclick="copyToClipboard(this)">{reference}</span> ';
        $html .= '<span class="label label-primary tag" onclick="copyToClipboard(this)">{firstname}</span> ';
        $html .= '<span class="label label-primary tag" onclick="copyToClipboard(this)">{lastname}</span> ';
        $html .= '<span class="label label-primary tag" onclick="copyToClipboard(this)">{email}</span> ';
        $html .= '<span class="label label-primary tag" onclick="copyToClipboard(this)">{amount}</span><br/><br/> ';
        $html .= '<span class="label label-primary tag" onclick="copyToClipboard(this)">{addressdelivery}</span> ';
        $html .= '<span class="label label-primary tag" onclick="copyToClipboard(this)">{addressinvoice}</span> ';
        $html .= '<span class="label label-primary tag" onclick="copyToClipboard(this)">{date}</span> ';
        $html .= '<span class="label label-success text_copied" style="display:none;">Tag Copied</span>';
        $html .= '</center><hr><div>';

        return $html;
    }

    /**
     * Set Placeholder Variables Values
     */        
    public function setVariableValues(
        $msg,
        $order,
        $customer,
        $delivery_address,
        $invoice_address
    ) {
        $precision = Configuration::get('PS_PRICE_DISPLAY_PRECISION') ? Configuration::get('PS_PRICE_DISPLAY_PRECISION') : "2";
        $order_amount = round((float) $order->total_paid, $precision);
        
        $date = date('d-m-Y',strtotime($order->date_add));

        $msg = str_replace("{id_order}", $order->id, $msg);
        $msg = str_replace("{reference}", $order->reference, $msg);
        $msg = str_replace("{firstname}", $customer->firstname, $msg);
        $msg = str_replace("{lastname}", $customer->lastname, $msg);
        $msg = str_replace("{email}", $customer->email, $msg);
        $msg = str_replace("{amount}", $order_amount, $msg);
        $msg = str_replace("{addressdelivery}", $delivery_address, $msg);
        $msg = str_replace("{addressinvoice}", $invoice_address, $msg);
        $msg = str_replace("{date}", $date, $msg);

        return $msg;
    }

    /**
     * Send SMS API Request using cURL
     */ 
    public function sendSMSRequest($phone, $message)
    {
        $API_AUTH_TOKEN = 'Authorization: '.Configuration::get('SMSADVERT_API_AUTH_TOKEN');
        $SMSADVERT_CHANNEL = (bool) Configuration::get('SMSADVERT_CHANNEL');
        $SMSADVERT_FAILOVER = Configuration::get('SMSADVERT_FAILOVER');
        $sendAsShort = $SMSADVERT_CHANNEL ? '"sendAsShort": true' : '"sendAsShort": false';
        $failover = ((!$SMSADVERT_CHANNEL) && $SMSADVERT_FAILOVER) ? ',"failover": "short"' : '';

        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => _API_URL,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS =>'{
            "phone": "'.$phone.'",
            "shortTextMessage": "'.$message.'",
            '.$sendAsShort.'
            '.$failover.'
            }',
            CURLOPT_HTTPHEADER => array(
                $API_AUTH_TOKEN,
                'Content-Type: application/json'
            ),
        ));

        $response = curl_exec($curl);

        curl_close($curl);
        
        return json_decode($response);
    }

    /**
     * Add Message log to database
     */        
    public function addMessageLog($phone, $msg, $msg_id)
    {
        if (isset($phone) && isset($msg) && isset($msg_id)) {
            Db::getInstance()->insert('smsadvert_log', [
                'msg_id'    =>  pSQL($msg_id),
                'phone'     =>  pSQL($phone),
                'message'   =>  pSQL($msg),
                'date_sent' =>  date("Y-m-d H:i:s")
            ]);
        }
    }
}
